/*-----------------------DROP---------------------*/
/*------------------------------------------------*/
/*
DROP TABLE Log;
DROP SEQUENCE Log_seq;
DROP TABLE Missatge;
DROP TABLE EstatSms;
DROP TABLE CanviEstat;
DROP TABLE EstatTicket;
DROP TABLE AssigTicketGrup;
DROP TABLE Ticket;
DROP TABLE CreacioTicket;
DROP TABLE Canal;
DROP TABLE TipusTicket;
DROP TABLE OfertaGrupServei;
DROP TABLE Servei;
DROP TABLE GrupServei;
DROP TABLE Prioritat;
DROP TABLE Tecnic;
DROP TABLE GrupSuport;
DROP TABLE NivellSuport;
DROP TABLE Interaccio;
DROP TABLE Usuari;
DROP TABLE Seu;
DROP TABLE Pais;
DROP TYPE va_params;
*/
/*----------------------CREATE--------------------*/
/*------------------------------------------------*/

CREATE TYPE va_params as VARRAY (10) OF VARCHAR2 (50 CHAR);
/

CREATE TABLE Pais (
	nom VARCHAR2 (50 CHAR) 	CONSTRAINT NN_Pais_nom NOT NULL,
	CONSTRAINT PK_Pais PRIMARY KEY (nom)
);

CREATE TABLE Seu (
	codi VARCHAR2 (20 CHAR) CONSTRAINT NN_Seu_codi NOT NULL,
	direccio VARCHAR2 (50 CHAR) CONSTRAINT NN_Seu_direccio NOT NULL,
	tipologia VARCHAR2 (50 CHAR),
	pais VARCHAR2 (50 CHAR) CONSTRAINT NN_Seu_pais NOT NULL,
	CONSTRAINT PK_Seu PRIMARY KEY (codi),
	CONSTRAINT FK_Seu_Pais FOREIGN KEY (pais) REFERENCES Pais (nom)
);

CREATE TABLE Usuari (
	mail VARCHAR2 (50 CHAR) CONSTRAINT NN_Usuari_mail NOT NULL,
	data_hora_reg TIMESTAMP CONSTRAINT NN_Usuari_data_hora_reg NOT NULL,
	data_hora_baixa TIMESTAMP,
	seu VARCHAR2 (20 CHAR) CONSTRAINT NN_Usuari_seu NOT NULL,
	CONSTRAINT PK_Usuari PRIMARY KEY (mail),
	CONSTRAINT FK_Usuari_Seu FOREIGN KEY (seu) REFERENCES Seu (codi),
	CONSTRAINT CH_data_reg_baixa CHECK (data_hora_baixa IS NULL 
		OR data_hora_baixa > data_hora_reg) /*RT4*/
);

CREATE TABLE Interaccio (
	usuari VARCHAR2 (50 CHAR) CONSTRAINT NN_Interaccio_usuari NOT NULL,
	data_hora TIMESTAMP CONSTRAINT NN_Interaccio_data_hora NOT NULL,
	accio VARCHAR2 (50 CHAR) CONSTRAINT NN_Interaccio_accio NOT NULL,
	CONSTRAINT PK_Interaccio PRIMARY KEY (usuari, data_hora),
	CONSTRAINT FK_Interaccio_Usuari FOREIGN KEY (usuari) REFERENCES Usuari (mail)
);

CREATE TABLE NivellSuport (
	nivell VARCHAR2 (20 CHAR) CONSTRAINT NN_NivellSuport_nivell NOT NULL,
	valor INTEGER CONSTRAINT NN_NivellSuport_valor NOT NULL,
	CONSTRAINT PK_NivellSuport PRIMARY KEY (nivell)
);

CREATE TABLE GrupSuport (
	codi VARCHAR2 (20 CHAR) CONSTRAINT NN_GrupSuport_codi NOT NULL,
	nom VARCHAR2 (50 CHAR) CONSTRAINT NN_GrupSuport_nom NOT NULL,
	nivell VARCHAR2 (20 CHAR) CONSTRAINT NN_GrupSuport_nivell NOT NULL,
	CONSTRAINT PK_GrupSuport PRIMARY KEY (codi),
	CONSTRAINT FK_GrupSuport_nivell FOREIGN KEY (nivell) REFERENCES NivellSuport (nivell)
);

CREATE TABLE Tecnic (
	mail VARCHAR2 (50 CHAR) CONSTRAINT NN_Tecnic_mail NOT NULL,
	grup VARCHAR2 (20 CHAR) CONSTRAINT NN_Tecnic_grup NOT NULL,
	CONSTRAINT PK_Tecnic PRIMARY KEY (mail),
	CONSTRAINT FK_Tecnic_Usuari FOREIGN KEY (mail) REFERENCES Usuari (mail),
	CONSTRAINT FK_Tecnic_Grup FOREIGN KEY (grup) REFERENCES GrupSuport (codi)
);

CREATE TABLE Prioritat (
	prioritat INTEGER CONSTRAINT NN_Prioritat_prioritat NOT NULL,
	CONSTRAINT PK_Prioritat PRIMARY KEY (prioritat)
);

CREATE TABLE GrupServei (
	grup VARCHAR2 (20 CHAR) CONSTRAINT NN_GrupServei_grup NOT NULL,
	CONSTRAINT PK_GrupServei PRIMARY KEY (grup)
);

CREATE TABLE Servei (
	codi VARCHAR2 (20 CHAR) CONSTRAINT NN_Servei_codi NOT NULL,
	descripcio VARCHAR2 (50 CHAR),
	grup VARCHAR2 (20 CHAR) CONSTRAINT NN_Servei_grup NOT NULL,
	temps_mig INTEGER CONSTRAINT NN_Servei_temps NOT NULL,
	prioritat INTEGER CONSTRAINT NN_Servei_prioritat NOT NULL,
	CONSTRAINT PK_Servei PRIMARY KEY (codi),
	CONSTRAINT FK_Servei_Grup FOREIGN KEY (grup) REFERENCES GrupServei (grup),
	CONSTRAINT FK_Servei_Prioritat FOREIGN KEY (prioritat) REFERENCES Prioritat (prioritat)
);

CREATE TABLE OfertaGrupServei (
	grup VARCHAR2 (20 CHAR) CONSTRAINT NN_OfertaGrupServei_grup NOT NULL,
	servei VARCHAR2 (20 CHAR) CONSTRAINT NN_OfertaGrupServei_servei NOT NULL,
	CONSTRAINT PK_OfertaGrupServei PRIMARY KEY (grup, servei),
	CONSTRAINT FK_OfertaGrupServei_Grup FOREIGN KEY (grup) REFERENCES GrupSuport (codi),
	CONSTRAINT FK_OfertaGrupServei_Servei FOREIGN KEY (servei) REFERENCES Servei (codi)
);

CREATE TABLE TipusTicket (
	tipus VARCHAR2 (20 CHAR) CONSTRAINT NN_TipusTicket_tipus NOT NULL,
	CONSTRAINT PK_TipusTicket PRIMARY KEY (tipus)
);

CREATE TABLE Canal (
	canal VARCHAR2 (20 CHAR) CONSTRAINT NN_Canal_canal NOT NULL,
	CONSTRAINT PK_Canal PRIMARY KEY (canal)
);

CREATE TABLE CreacioTicket (
	metode VARCHAR2 (20 CHAR) CONSTRAINT NN_CreacioTicket_metode NOT NULL,
	CONSTRAINT PK_CreacioTicket PRIMARY KEY (metode)
);

CREATE TABLE Ticket (
	numero VARCHAR2 (20 CHAR) CONSTRAINT NN_Ticket_numero NOT NULL,
	data_hora_creacio TIMESTAMP CONSTRAINT NN_Ticket_data_hora_creacio NOT NULL,
	servei VARCHAR2 (20 CHAR) CONSTRAINT NN_Ticket_servei NOT NULL,
	prioritat INTEGER CONSTRAINT NN_Ticket_prioritat NOT NULL,
	tipus VARCHAR2 (20 CHAR) CONSTRAINT NN_Ticket_tipus NOT NULL,
	canal VARCHAR2 (20 CHAR) CONSTRAINT NN_Ticket_canal NOT NULL,
	creacio VARCHAR2 (20 CHAR) CONSTRAINT NN_Ticket_creacio NOT NULL,
	usuari VARCHAR2 (50 CHAR) CONSTRAINT NN_Ticket_usuari NOT NULL,
	CONSTRAINT PK_Ticket PRIMARY KEY (numero),
	CONSTRAINT FK_Ticket_Servei FOREIGN KEY (servei) REFERENCES Servei (codi),
	CONSTRAINT FK_Ticket_Prioritat FOREIGN KEY (prioritat) REFERENCES Prioritat (prioritat),
	CONSTRAINT FK_Ticket_TipusTicket FOREIGN KEY (tipus) REFERENCES TipusTicket (tipus),
	CONSTRAINT FK_Ticket_Canal FOREIGN KEY (canal) REFERENCES Canal (canal),
	CONSTRAINT FK_Ticket_CreacioTicket FOREIGN KEY (creacio) REFERENCES CreacioTicket (metode),
	CONSTRAINT FK_Ticket_Usuari FOREIGN KEY (usuari) REFERENCES Usuari (mail)
);

CREATE TABLE AssigTicketGrup (
	ticket VARCHAR2 (20 CHAR) CONSTRAINT NN_AssigTicketGrup_ticket NOT NULL,
	grup VARCHAR2 (20 CHAR) CONSTRAINT NN_AssigTicketGrup_grup NOT NULL,
	data_hora TIMESTAMP CONSTRAINT NN_AssigTicketGrup_data NOT NULL,
	CONSTRAINT PK_AssigTicketGrup PRIMARY KEY (ticket, grup),
	CONSTRAINT FK_AssigTicketGrup_Ticket FOREIGN KEY (ticket) REFERENCES Ticket (numero),
	CONSTRAINT FK_AssigTicketGrup_GrupSuport FOREIGN KEY (grup) REFERENCES GrupSuport (codi)
	/*RT1 ha de controlar-se al procediment d'alta d'una nova assignacio*/
	/*RT5 ha de controlar-se al procediment d'alta d'una nova assignacio*/
);

CREATE TABLE EstatTicket (
	estat VARCHAR2 (20 CHAR) CONSTRAINT NN_EstatTicket_estat NOT NULL,
	CONSTRAINT PK_EstatTicket PRIMARY KEY (estat)
);

CREATE TABLE CanviEstat (
	ticket VARCHAR2 (20 CHAR) CONSTRAINT NN_CanviEstat_ticket NOT NULL,
	data_hora TIMESTAMP CONSTRAINT NN_CanviEstat_data NOT NULL,
	tecnic VARCHAR2 (50 CHAR) CONSTRAINT NN_CanviEstat_tecnic NOT NULL,
	estat VARCHAR2 (20 CHAR) CONSTRAINT NN_CanviEstat_estat NOT NULL,
	CONSTRAINT PK_CanviEstat PRIMARY KEY (ticket, data_hora),
	CONSTRAINT FK_CanviEstat_ticket FOREIGN KEY (ticket) REFERENCES Ticket (numero),
	CONSTRAINT FK_CanviEstat_tecnic FOREIGN KEY (tecnic) REFERENCES Tecnic (mail),
	CONSTRAINT FK_CanviEstat_estat FOREIGN KEY (estat) REFERENCES EstatTicket (estat)
	/*RT2 ha de controlar-se al procediment d'alta d'un nou canvi d'estat*/
);

CREATE TABLE EstatSms (
	estat VARCHAR2 (20 CHAR) CONSTRAINT NN_EstatSms_estat NOT NULL,
	CONSTRAINT PK_EstatSms PRIMARY KEY (estat)
);

CREATE TABLE Missatge (
	codi VARCHAR2 (20 CHAR) CONSTRAINT NN_Missatge_codi NOT NULL,
	data_hora TIMESTAMP CONSTRAINT NN_Missatge_data NOT NULL,
	emissor VARCHAR2 (50 CHAR) CONSTRAINT NN_Missatge_emissor NOT NULL,
	estat VARCHAR2 (20 CHAR) CONSTRAINT NN_Missatge_estat NOT NULL,
	ticket VARCHAR2 (20 CHAR) CONSTRAINT NN_Missatge_ticket NOT NULL,
	CONSTRAINT PK_Missatge PRIMARY KEY (codi),
	CONSTRAINT FK_Missatge_estat FOREIGN KEY (estat) REFERENCES EstatSms (estat),
	CONSTRAINT FK_Missatge_ticket FOREIGN KEY (ticket) REFERENCES Ticket (numero)
	/*RT3 ha de controlar-se al procediment d'alta d'un nou missatge*/
);
 
CREATE SEQUENCE Log_seq;
 
CREATE TABLE Log (
	codi NUMBER CONSTRAINT NN_Log_codi NOT NULL,
	data_hora TIMESTAMP CONSTRAINT NN_Log_data NOT NULL,
	procediment VARCHAR2 (50 CHAR) CONSTRAINT NN_Log_procediment NOT NULL,
	param_in va_params CONSTRAINT NN_Log_param_in NOT NULL,
	RSP VARCHAR2 (200 CHAR) CONSTRAINT NN_Log_RSP NOT NULL,
	CONSTRAINT PK_Log PRIMARY KEY (codi)
);